#!/usr/bin/env python
# -*- coding: utf-8 -*-
# author: Pocholo
# date: 3/5/21

#Inspire by Conbagui tutorials found here: https://gimpnexuxtutorials.blogspot.com/2021/03/how-to-create-coral-texture-and-coral.html
#                                          https://gimpnexuxtutorials.blogspot.com/2016/05/underwater-scene-tutorial.html

# Comments directed to http://gimpchat.com

# Installation:
# This script should be placed in the user plugins folder

# Copyright (C)

#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by   
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY# without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <https://www.gnu.org/licenses/>.   

from gimpfu import *
import gettext
import os, sys,time 

gettext.install("gimp20", gimp.locale_directory, unicode=True)

def pm_create_an_underwater_text(customtext, fontname, size):
          
    #Initiates the temporary state
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
        
    #Background and noise combined
    img = pdb.gimp_image_new(1920, 1200, RGB_IMAGE)   
    backLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Background", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, backLayer, 0)
    pdb.gimp_layer_add_alpha(backLayer)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_drawable_fill(backLayer, FILL_FOREGROUND)
    pdb.gimp_context_set_gradient("Bottom of the sea")
    pdb.gimp_drawable_edit_gradient_fill(backLayer,                                  
                                                  0,  #gradient type - linear         
                                                  0,  #offset
                                               TRUE,  #supe-samle
                                                  3,  #supersample_max_depth
                                               0.20,  #threshold 
                                               TRUE,  #dither                                                 
                                                960,  #x1
                                                  0,  #y1
                                                960,  #x2
                                               1150)  #y2
   	
    pdb.gimp_display_new(img)
    
    #Make the sand horizon, wavy
    pdb.plug_in_ripple(img, backLayer, 
                                  200,   #period 
                                    2,   #amplitude
                                    0,   #orientation - vertical
                                    1,   #edges - wrap
                                    1,   #waveform - sine
                                 TRUE,   #antialias
                                FALSE)   #tile
    pdb.plug_in_gauss(img, backLayer, 30, 30, 1)
    
    #Water surface
    surfLayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Water surface", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_add_layer(img, surfLayer, 0)
    pdb.gimp_drawable_fill(surfLayer, FILL_TRANSPARENT)
    pdb.gimp_layer_add_alpha(surfLayer)
    pdb.gimp_image_select_rectangle(img, 2, 0, 0, 1920, 300)
    pdb.gimp_context_set_foreground("White")
    pdb.gimp_edit_bucket_fill(surfLayer, BUCKET_FILL_FG, LAYER_MODE_NORMAL, 100, 255, FALSE, 0, 0)
    pdb.plug_in_solid_noise(img, surfLayer, FALSE, FALSE, 5, 15, 16, 16)
    pdb.gimp_selection_none(img)
    
    #Make active the image and save to channel      
    channel = pdb.gimp_selection_save(img)   
       
    #Create an image duplicate
    non_empty = pdb.gimp_edit_copy(surfLayer)
    newlayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "New layer", 100, LAYER_MODE_NORMAL)
    pdb.gimp_image_insert_layer(img, newlayer, None, 0)
    pdb.gimp_layer_add_alpha(newlayer)
    newlayer = pdb.gimp_edit_paste(newlayer, FALSE)
    pdb.gimp_floating_sel_anchor(newlayer) 
    newlayer = pdb.gimp_image_get_layer_by_name(img, "New layer")
    pdb.gimp_item_set_name(newlayer, "Wrap effect")
    
    #Desaturate the image
    pdb.gimp_drawable_desaturate(newlayer, DESATURATE_LIGHTNESS)
    
    #Blur the image
    pdb.plug_in_gauss_iir2(img, newlayer, 15, 15)
    
    #Apply edge filter
    pdb.plug_in_edge(img, newlayer, 2.0, 1, 4)
    
    #Invert the image
    pdb.gimp_invert(newlayer)
      
    #Create a layer mask
    newMask = pdb.gimp_layer_create_mask(newlayer, ADD_MASK_BLACK)
    pdb.gimp_layer_add_mask(newlayer, newMask)
   
    #Copy the image and paste it on the layer mask
    non_empty = pdb.gimp_edit_copy(newlayer)
    newMaskk = pdb.gimp_edit_paste(newMask, FALSE)
    pdb.gimp_floating_sel_anchor(newMaskk) 
     
    #Apply levels to the image and the layer mask   
    pdb.gimp_levels(newlayer, HISTOGRAM_VALUE, 0, 255, .40, 0, 255)
    pdb.gimp_levels(newlayer, HISTOGRAM_VALUE, 0, 255, .40, 0, 255)
    pdb.gimp_levels(newMask, HISTOGRAM_VALUE, 0, 255, .40, 0, 255)
    pdb.gimp_levels(newMask, HISTOGRAM_VALUE, 0, 255, .40, 0, 255)

    #Apply layer mask and merge the layer
    pdb.gimp_layer_remove_mask(newlayer, MASK_APPLY)
    newlayer = pdb.gimp_image_merge_down(img, newlayer, CLIP_TO_IMAGE)
    
    #Remove the channel and set active the layer
    pdb.gimp_image_remove_channel(img, channel)  
    pdb.gimp_image_set_active_layer(img, newlayer)
       
    #Apply perspective
    pdb.gimp_image_select_item(img, 2, newlayer)
    newlayer = pdb.gimp_item_transform_perspective(newlayer, 
                                                      -2000,  #x0 
                                                      -1600,  #y0 
                                                       4000,  #x1 
                                                      -1600,  #y1 
                                                          0,  #x2 
                                                        300,  #y2 
                                                       1920,  #x3 
                                                        300)  #y3 
                                                  
    
    pdb.gimp_floating_sel_anchor(newlayer)
    pdb.gimp_selection_none(img)
    surfLayer = pdb.gimp_image_get_layer_by_name(img, "Water surface")
    pdb.gimp_layer_resize_to_image_size(surfLayer)
       
    #Make water distance blur
    pdb.gimp_image_select_item(img, 2, surfLayer)
    pdb.gimp_selection_invert(img)
    pdb.gimp_selection_grow(img, 25)
    pdb.gimp_selection_feather(img, 400)
    pdb.gimp_drawable_edit_clear(surfLayer)
    pdb.gimp_drawable_edit_clear(surfLayer)
    pdb.gimp_drawable_edit_clear(surfLayer)
    pdb.gimp_drawable_edit_clear(surfLayer)
    pdb.gimp_drawable_edit_clear(surfLayer)
    pdb.gimp_image_select_item(img, 2, surfLayer) 
    surfLayer = pdb.gimp_item_transform_scale(surfLayer, -1525, -230, 3464, 535) 
    pdb.gimp_floating_sel_anchor(surfLayer)
    pdb.gimp_selection_none(img)
    surfLayer = pdb.gimp_image_get_layer_by_name(img, "Water surface")
    pdb.gimp_layer_resize_to_image_size(surfLayer)
    
    
    #Create an layer duplicate
    non_empty = pdb.gimp_edit_copy(surfLayer)
    rayslayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Sunlight rays", 80, LAYER_MODE_SCREEN)
    pdb.gimp_image_insert_layer(img, rayslayer, None, 0)
    pdb.gimp_layer_add_alpha(rayslayer)
    rayslayer = pdb.gimp_edit_paste(rayslayer, FALSE)
    pdb.gimp_floating_sel_anchor(rayslayer) 
    rayslayer = pdb.gimp_image_get_layer_by_name(img, "Sunlight rays")
    pdb.gimp_layer_set_mode(surfLayer, LAYER_MODE_LINEAR_LIGHT)
    pdb.plug_in_luce(img, rayslayer, 
                                  1, #point_light - true
                                  0, #quadratic_attenuation - false
                                  1, #zero_mode - true
                                  1, #add_source - true
                                  1, #source_zero- true
                                  1, #source_positive_normalize - true
                                  1, #source_negative_normalize- true
                                  1, #positive_normalize - true
                                  1, #negative_normalize - true
                                  1, #num_threads_to_use
                               1.10, #positive_intensity - in feet
                               1.10, #negative_intensity - in feet
                              0.55, #center_x - in feet
                              -0.58, #center_y - in feet
                                0.1, #direction_x
                                0.1, #direction_y
                                0.0, #zeros0
                                0.0, #zeros1
                                0.0, #zeros2
                                0.0, #zeros3
                                0.0) #zeros4
    
    #Make a sunlight reflection
    non_empty = pdb.gimp_edit_copy(surfLayer)
    reflectionlayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Sunlight reflection", 80, LAYER_MODE_SOFTLIGHT)
    pdb.gimp_image_insert_layer(img, reflectionlayer, None, 2)
    pdb.gimp_layer_add_alpha(reflectionlayer)
    reflectionlayer = pdb.gimp_edit_paste(reflectionlayer, FALSE)
    pdb.gimp_floating_sel_anchor(reflectionlayer) 
    reflectionlayer = pdb.gimp_image_get_layer_by_name(img, "Sunlight reflection")
    reflectionlayer = pdb.gimp_item_transform_flip_simple(reflectionlayer, ORIENTATION_VERTICAL, TRUE, 0) 
    reflectionlayer = pdb.gimp_image_merge_down(img, reflectionlayer, CLIP_TO_IMAGE)
   
   #Create the text
    backLayer = pdb.gimp_image_get_layer_by_name(img, "Background")	
    textW = size/0.3
    textH = size/1
    pdb.gimp_context_set_foreground((0, 0, 0))
    textLayer = pdb.gimp_text_fontname(img, None, 0, 0, customtext, 10, 1, size, PIXELS, fontname)
    pdb.gimp_text_layer_set_justification(textLayer, 2)
    pdb.gimp_text_layer_set_letter_spacing(textLayer, -30)
   
        
    #Resize text
    pdb.gimp_layer_scale(textLayer, textW, textH, TRUE)
                
    #Making the text centered	
    textLayer = pdb.gimp_image_get_active_layer(img)
    pdb.gimp_image_set_active_layer(img, textLayer)
    wh = pdb.gimp_drawable_width(textLayer)
    hh = pdb.gimp_drawable_height(textLayer)
    w = pdb.gimp_image_width(img)
    h = pdb.gimp_image_height(img)
    pdb.gimp_image_set_active_layer(img,textLayer)
    centerx=w/2
    centery=h/2
    adw=wh/2
    adh=hh/2
    sw=centerx - adw
    sh=centery - adh		
    pdb.gimp_layer_set_offsets(textLayer, sw, sh)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    
    #Rename the text layer 
    pdb.gimp_item_set_name(textLayer, "Text")
         
    #Text inner shadow
    pdb.python_layerfx_inner_shadow(img, textLayer, 
                                         (0, 0, 0),   #color
                                                70,   #opacity
                                                 0,   #contou linear
                                                 0,   #noise
                                                 0,   #mode for normal
                                                 1,   #source
                                                 0,   #choke 
                                                20,   #size 
                                               -90,   #offset_angle 
                                                20,   #offset_distance 
                                              FALSE)  #merge
    innershadow = pdb.gimp_image_get_layer_by_name(img, "Text-innershadow")
    
    #Inner Glow								   
    pdb.python_layerfx_inner_glow(img, textLayer, 
									 (121, 0, 0),   #color
											  70,   #opacity
									 		   0,   #contou
											   0,   #noise
								              17,   #mode - burn
											   1,   #source
											   0,   #choke 
											  85,   #size 
										   FALSE)   #merge
    innerglow = pdb.gimp_image_get_layer_by_name(img, "Text-innerglow")
    
    #Bevel and emboss
    pdb.python_layerfx_bevel_emboss(img, textLayer, 
                                                 1,   #style inner bevel
                                                65,   #depth 
                                                 0,   #direction 
                                                27,   #size 
                                                 1,   #soften
                                                90,   #angle 
                                                30,   #altitude 
                                                 3,   #gloss_contour - cove deep 
                                   (255, 218, 178),   #highlight_color 
                                                 4,   #highlight_mode screen
                                                65,   #highlight_opacity
                                      (50, 40, 32),   #shadow_color 
                                                 3,   #shadow_mode - multyply
                                                40,   #shadow_opacity 
                                                 0,   #surface_contour - linear
                                              TRUE,   #use_texture 
                                       "Coral.jpg",   #pattern 
                                               100,   #scale 
                                                50,   #tex_depth 
                                              TRUE,   #invert 
                                             FALSE)   #merge    
    bHighlight = pdb.gimp_image_get_layer_by_name(img, "Text-highlight")
    bShadow = pdb.gimp_image_get_layer_by_name(img, "Text-shadow")
    
    #Satin
    pdb.python_layerfx_satin(img, textLayer, 
                                (220, 0, 5),   #color
                                         50,   #opacity
                                         14,   #mode - value
                                         19,   #offset_angle 
                                         30,   #offset_distance
                                         17,   #size 
                                          0,   #contou - linear
                                       TRUE,   #invert 
                                      FALSE)   #merge
    satin = pdb.gimp_image_get_layer_by_name(img, "Text-satin")                                  
    
    #Color overlay
    pdb.python_layerfx_color_overlay(img, textLayer, 
                                       (215, 37, 0),   #color
                                                100,   #opacity
                                                 13,   #mode - color   
                                              FALSE)   #merge
    color = pdb.gimp_image_get_layer_by_name(img, "Text-color")                                         
    
    #Pattern overlay		
    pdb.python_layerfx_pattern_overlay(img, textLayer, 
                                          "Coral.jpg",   #pattern
                                                  100,   #opacity
                                                    0,   #mode grain merge
                                                  100,   #scale
                                                    0,   #interpolation
                                                FALSE)   #merge
    pattern = pdb.gimp_image_get_layer_by_name(img, "Text-pattern")                                             
    
    #Create a new from visible image
    pdb.gimp_item_set_visible(backLayer, FALSE)
    laidTLayer = pdb.gimp_layer_new_from_visible(img, img, "Laid Text")
    pdb.gimp_image_add_layer(img, laidTLayer, 9)
    pdb.gimp_layer_set_mode(laidTLayer, LAYER_MODE_NORMAL_LEGACY)
    pdb.gimp_item_set_visible(backLayer, TRUE)
   
    pdb.gimp_item_set_visible(innershadow, FALSE)  
    pdb.gimp_item_set_visible(innerglow, FALSE)
    pdb.gimp_item_set_visible(bHighlight, FALSE)
    pdb.gimp_item_set_visible(bShadow, FALSE)
    pdb.gimp_item_set_visible(satin, FALSE)
    pdb.gimp_item_set_visible(color, FALSE)
    pdb.gimp_item_set_visible(pattern, FALSE)
    
    
    #using map rotate
    pdb.plug_in_map_object(img, textLayer, 
									    0,    # Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
							0.5, 0.5, 2.0,    # viewpoint
							0.5, 0.5, 0.0,    # object pos
							1.0, 0.0, 0.0,    # first axis
							0.0, 1.0, 0.0,    # 2nd axis
						  0.0, 0.0, -60.0,    # axis rotation
										2,    # Type of lightsource (0=point,1=directional,2=none)
						  (255, 255, 255),    # Lightsource color (r,g,b)
						  -0.5, -0.5, 2.0,    # light position
						  -1.0, -1.0, 1.0,    # light direction
			 1.00, 0.60, 0.30, 1.50, 50.0,    # material (amb, diff, refl, spec, high)
									 True,    # antialias
									False,    # tile
									False,    # new image
									 True,    # transparency
									 0.25,    # Sphere/cylinder radius (only used when maptype=1 or 3)
							0.5, 0.5, 0.5,    # box size
									 1.00,    # unused parameter Cylinder length
									 None,    # Box front face (set these to -1 if not used)
									 None,    # Box back face
									 None,    # Box top face									
									 None,    # Box bottom face
									 None,    # Box left face
									 None,    # Box right face
									 None,    # Cylinder top face (unused parameter)
									 None)    # Cylinder bottom face (unu
    
     #using map rotate
    pdb.plug_in_map_object(img, laidTLayer, 
									    0,    # Type of mapping (0=plane,1=sphere,2=box,3=cylinder)
							0.5, 0.5, 2.0,    # viewpoint
							0.5, 0.5, 0.0,    # object pos
							1.0, 0.0, 0.0,    # first axis
							0.0, 1.0, 0.0,    # 2nd axis
						  0.0, 0.0, -60.0,    # axis rotation
										2,    # Type of lightsource (0=point,1=directional,2=none)
						  (255, 255, 255),    # Lightsource color (r,g,b)
						  -0.5, -0.5, 2.0,    # light position
						  -1.0, -1.0, 1.0,    # light direction
			 1.00, 0.60, 0.30, 1.50, 50.0,    # material (amb, diff, refl, spec, high)
									 True,    # antialias
									False,    # tile
									False,    # new image
									 True,    # transparency
									 0.25,    # Sphere/cylinder radius (only used when maptype=1 or 3)
							0.5, 0.5, 0.5,    # box size
									 1.00,    # unused parameter Cylinder length
									 None,    # Box front face (set these to -1 if not used)
									 None,    # Box back face
									 None,    # Box top face									
									 None,    # Box bottom face
									 None,    # Box left face
									 None,    # Box right face
									 None,    # Cylinder top face (unused parameter)
									 None)    # Cylinder bottom face (unu
    
    #Move the layer to the bottom of the canvas   
    pdb.gimp_layer_set_linked(textLayer, TRUE)
    pdb.gimp_layer_set_linked(laidTLayer, TRUE)
    laidTLayer = pdb.gimp_item_transform_translate(laidTLayer, 0, 492)
    pdb.gimp_layer_resize_to_image_size(textLayer)
    pdb.gimp_layer_resize_to_image_size(laidTLayer)
    
    #Plexi text drop shadow
    pdb.python_layerfx_drop_shadow(img, laidTLayer, 
                                       ( 0, 0, 0),   #color
                                              100,   #opacity
                                                0,   #contou - linear
                                                0,   #noise 
                                                3,   #mode for multiply 
                                                0,   #spread 
                                               10,   #size 
                                               90,   #offset_angle 
                                               10,   #offset_distance 
                                             TRUE,   #knockout 
                                            FALSE)   #merge
    
    #Text inner shadow
    pdb.python_layerfx_inner_shadow(img, laidTLayer, 
                                         (0, 0, 0),   #color
                                                70,   #opacity
                                                 0,   #contou linear
                                                 0,   #noise
                                                 0,   #mode for normal
                                                 1,   #source
                                                 0,   #choke 
                                                20,   #size 
                                               -90,   #offset_angle 
                                                20,   #offset_distance 
                                              FALSE)  #merge
    
    #Bevel and emboss
    pdb.python_layerfx_bevel_emboss(img, laidTLayer, 
                                                 3,   #style inner bevel
                                                65,   #depth 
                                                 0,   #direction 
                                                27,   #size 
                                                16,   #soften
                                                90,   #angle 
                                                30,   #altitude 
                                                 3,   #gloss_contour - cove deep 
                                   (255, 218, 178),   #highlight_color 
                                                 4,   #highlight_mode screen
                                                50,   #highlight_opacity
                                      (50, 40, 32),   #shadow_color 
                                                 3,   #shadow_mode - multyply
                                                40,   #shadow_opacity 
                                                 0,   #surface_contour - linear
                                             FALSE,   #use_texture 
                                       "Coral.jpg",   #pattern 
                                               100,   #scale 
                                                50,   #tex_depth 
                                              TRUE,   #invert 
                                             FALSE)   #merge    
    
    #Make a sunlight reflection
    non_empty = pdb.gimp_edit_copy(surfLayer)
    reflectionlayer = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Sunlight reflection", 100, LAYER_MODE_ADDITION)
    pdb.gimp_image_insert_layer(img, reflectionlayer, None, 2)
    pdb.gimp_layer_add_alpha(reflectionlayer)
    reflectionlayer = pdb.gimp_edit_paste(reflectionlayer, FALSE)
    pdb.gimp_floating_sel_anchor(reflectionlayer) 
    reflectionlayer = pdb.gimp_image_get_layer_by_name(img, "Sunlight reflection")
    reflectionlayer = pdb.gimp_item_transform_flip_simple(reflectionlayer, ORIENTATION_VERTICAL, TRUE, 0) 
    
    #Make a sunlight reflection
    non_empty = pdb.gimp_edit_copy(reflectionlayer)
    reflectionlayer2 = pdb.gimp_layer_new(img, img.width, img.height, RGBA_IMAGE, "Sunlight reflection 2", 65, LAYER_MODE_SCREEN)
    pdb.gimp_image_insert_layer(img, reflectionlayer2, None, 3)
    pdb.gimp_layer_add_alpha(reflectionlayer2)
    reflectionlayer2 = pdb.gimp_edit_paste(reflectionlayer2, FALSE)
    pdb.gimp_floating_sel_anchor(reflectionlayer2) 
    reflectionlayer2 = pdb.gimp_image_get_layer_by_name(img, "Sunlight reflection 2")
      
	#Set gimp to default
    pdb.gimp_context_set_defaults()
    pdb.gimp_context_set_default_colors()
   
	
register(
    "pm_create_an_underwater_text",
    "Creates an underwater text effect",
    "Creates an underwater text effect",
    "Pocholo",
    "Pocholo",
    "2021",
    "Create an underwater text",
    "",
    [        
    (PF_STRING, "customtext", "Text string", 'REEF'),
    (PF_FONT, "font", "Font name", "TANTOR"),
    (PF_SPINNER, "size", "Text size", 375, (375, 400, 1)),
    ],
    [],
    pm_create_an_underwater_text, menu="<Image>/Pocholo-scripts/Create an underwater text",
    domain=("gimp20", gimp.locale_directory))
main()					